part of pdf;

/// Implements a collection of named destinations in the document.
class PdfNamedDestinationCollection implements _IPdfWrapper {
  /// Initializes a new instance of the [PdfNamedDestinationCollection] class.
  PdfNamedDestinationCollection() : super() {
    _initialize();
  }

  /// Collection of the named destinations.
  final List<PdfNamedDestination> _namedCollections = <PdfNamedDestination>[];

  /// Internal variable to store dictinary.
  final _PdfDictionary _dictionary = _PdfDictionary();

  /// Array of the named destinations.
  final _PdfArray _namedDestination = _PdfArray();

  /// Gets number of the elements in the collection.
  int get count => _namedCollections.length;

  /// Gets the [PdfNamedDestination] at the specified index.
  PdfNamedDestination operator [](int index) {
    if (index < 0 || index > count - 1) {
      throw RangeError('$index, Index is out of range.');
    }
    return _namedCollections[index];
  }

  /// Creates and adds a named destination.
  void add(PdfNamedDestination namedDestination) {
    ArgumentError.checkNotNull(
        namedDestination, 'The named destination value can\'t be null');
    _namedCollections.add(namedDestination);
  }

  /// Determines whether the specified named destinations presents in the collection.
  bool contains(PdfNamedDestination namedDestination) {
    ArgumentError.checkNotNull(
        namedDestination, 'The named destination value can\'t be null');
    return _namedCollections.contains(namedDestination);
  }

  /// Remove the specified named destination from the document.
  void remove(String title) {
    ArgumentError.checkNotNull(title, 'The title can\'t be null');
    int index = -1;
    for (int i = 0; i < _namedCollections.length; i++) {
      if (_namedCollections[i].title == title) {
        index = i;
        break;
      }
    }
    removeAt(index);
  }

  /// Remove the specified named destination from the document.
  void removeAt(int index) {
    if (index >= _namedCollections.length) {
      throw RangeError(
          'The index value should not be greater than or equal to the count.');
    }
    _namedCollections.removeAt(index);
  }

  /// Removes all the named destination from the collection.
  void clear() {
    _namedCollections.clear();
  }

  void insert(int index, PdfNamedDestination namedDestination) {
    ArgumentError.checkNotNull(
        namedDestination, 'The named destination value can\'t be null');
    if (index < 0 || index > count) {
      throw RangeError(
          'The index can\'t be less then zero or greater then Count.');
    }
    _namedCollections.insert(index, namedDestination);
  }

  /// Initializes instance.
  void _initialize() {
    _dictionary._beginSave = (Object sender, _SavePdfPrimitiveArgs ars) {
      for (PdfNamedDestination values in _namedCollections) {
        _namedDestination._add(_PdfString(values.title));
        _namedDestination._add(_PdfReferenceHolder(values));
      }
      _dictionary.setProperty(
          _DictionaryProperties.names, _PdfReferenceHolder(_namedDestination));
    };
  }

  /// Gets the element.
  @override
  _IPdfPrimitive get _element => _dictionary;

  set _element(_IPdfPrimitive value) {
    throw ArgumentError();
  }
}
