part of pdf;

/// The class provides methods and properties to handle the collections of [PdfPageLayer].
class PdfPageLayerCollection extends PdfObjectCollection {
  //Constructor
  /// Initializes a new instance of the [PdfPageLayerCollection] class with PDF page.
  PdfPageLayerCollection(PdfPage page) : super() {
    ArgumentError.checkNotNull(page, 'page');
    _page = page;
    _parseLayers(page);
  }

  //Fields
  PdfPage _page;

  //Properties
  /// Gets [PdfPageLayer] by its index from [PdfPageLayerCollection].
  PdfPageLayer operator [](int index) => _returnValue(index);
  PdfPageLayer _returnValue(int index) {
    return _list[index] as PdfPageLayer;
  }

  //Public methods
  /// Creates a new <see cref="PdfPageLayer"/> and adds it to the end of the collection.
  PdfPageLayer add() {
    final PdfPageLayer layer = PdfPageLayer(_page);
    layer._name = '';
    addLayer(layer);
    return layer;
  }

  /// Adds [PdfPageLayer] to the collection.
  int addLayer(PdfPageLayer layer) {
    ArgumentError.checkNotNull(layer, 'layer');
    if (layer.page != _page) {
      ArgumentError.value(layer, 'The layer belongs to another page');
    }
    _list.add(layer);
    final int listIndex = count - 1;
    _page._contents._add(_PdfReferenceHolder(layer));
    return listIndex;
  }

  /// Returns index of the [PdfPageLayer] in the collection if exists, -1 otherwise.
  int indexOf(PdfPageLayer layer) {
    ArgumentError.checkNotNull(layer, 'layer');
    return _list.indexOf(layer);
  }

  //Implementation
  void _parseLayers(PdfPage page) {
    ArgumentError.checkNotNull(page, 'page');
    final _PdfArray contents = page._contents;
    final _PdfStream saveStream = _PdfStream();
    final _PdfStream restoreStream = _PdfStream();
    const int saveState = 113;
    const int restoreState = 81;
    saveStream._dataStream = <int>[saveState];
    if (contents.count > 0) {
      contents._insert(0, _PdfReferenceHolder(saveStream));
    } else {
      contents._add(_PdfReferenceHolder(saveStream));
    }
    restoreStream._dataStream = <int>[restoreState];
    contents._add(_PdfReferenceHolder(restoreStream));
  }
}
